import os
import pandas as pd
import pvlib
from ephem import NeverUpError, AlwaysUpError
from datetime import datetime, timedelta

wd = '/data/inca_vol1/input_data/crop_pollination/input_intermediate/Agri4Cast_preprocessed/'

os.chdir(wd)
print(os.getcwd())

## Input file: "grid.csv" 
sites = pd.read_csv("/data/inca_vol1/input_data/crop_pollination/input_intermediate/Agri4Cast_preprocessed/grid.csv")
print(sites.head())
l = len(sites['LATITUDE'])
print(l)

l_month = pd.read_csv("/data/inca_vol1/input_data/crop_pollination/input_raw/Agri4Cast/MonthLength.csv")
print(l_month.head())

df = pd.DataFrame()



grid = sites['GRID_NO']
times = pd.date_range(start='2000-01-01', end='2000-12-31', tz='UTC', freq='D')#, closed='left')
for i in range(len(sites)):
    lat = sites['LATITUDE'][i]
    lon = sites['LONGITUDE'][i]
    grid = sites['GRID_NO'][i]
    print(f"GRIDCELL Number = {i + 1} of {l}, LAT = {lat}")

    dl = []

    # Calculate sunrise and sunset times
    solpos = pvlib.solarposition.get_solarposition(times, lat, lon)
    try:
        sunrise_sunset = pvlib.location.Location(lat,lon).get_sun_rise_set_transit(times)
        # Calculate day length
        day_length = (sunrise_sunset['sunset'] - sunrise_sunset['sunrise']).dt.total_seconds() / 3600
    except:
        #try to loop over the errors to fix them: remove no sun or only sun
        day_length = pd.Series()
        sunrise_sunset = pd.DataFrame()
        for time in times:
            try:
                sunrise_sunset = pvlib.location.Location(lat,lon).get_sun_rise_set_transit(pd.DatetimeIndex([pd.to_datetime(time)]))
                day_length_sub = (sunrise_sunset['sunset'] - sunrise_sunset['sunrise']).dt.total_seconds() / 3600
            except NeverUpError:
                day_length_sub = pd.Series([0], index =  pd.DatetimeIndex([pd.to_datetime(time)]))
            except AlwaysUpError as e :
                day_length_sub = pd.Series([24], index =  pd.DatetimeIndex([pd.to_datetime(time)]))

            day_length= pd.concat([day_length, day_length_sub])

    day_length.loc[ day_length < 0] =  day_length.loc[day_length < 0] +24

    avg_dl =  day_length.groupby(day_length.index.month).mean()



    dl_df = pd.DataFrame({
        "Month_Code": l_month['Month_Code'].to_list(),
        "GRID_NO": [grid] * 12,
        "Latitude": [lat] * 12,
        "Longitude": [lon] * 12,
        "Length": l_month['Length'].to_list(),
        "avg_daylength": avg_dl.to_list()
    })

    df = pd.concat([df, dl_df], ignore_index=True)

filename = "avg_daylength.csv"
colnames = ["Month_Code", "GRID_NO", "Latitude", "Longitude", "Length", "avg_daylength"]
df.to_csv(filename, index=False, sep=",", header=colnames)
print("end")

import pandas as pd
import pvlib

# Load latitude and longitude data
sites = pd.read_csv("/data/inca_vol1/input_data/crop_pollination/input_intermediate/Agri4Cast_preprocessed/grid.csv")



# Save the DataFrame to a CSV file
sites.to_csv("day_length.csv", index=False)