#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2022 European Union.
#
# The tool was developed with the contribution of the Joint Research Centre of the European Commission.
#
# This program is free software: you can redistribute it and/or modify it under the terms of the European Union Public
# Licence, either version 1.2 of the License, or (at your option) any later version.
# You may not use this work except in compliance with the Licence.
#
# You may obtain a copy of the Licence at: https://joinup.ec.europa.eu/collection/eupl/eupl-guidelines-faq-infographics
#
# Unless required by applicable law or agreed to in writing, software distributed under the Licence is distributed on
# an "AS IS" basis, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the Licence for the specific language governing permissions and limitations under the Licence.


"""
SYNOPSIS
    run_EBM_calculation.py [-v,--verbose] [-h,--help] [--version]

DESCRIPTION
    This snippet is for running the EBM model in preparation of the crop pollination model run.

PREREQUISITES
    Python => 3.7.0

EXAMPLES
    python -m inca.croppollination.preprocessing.run_EBM_calculation -i <file_path> -o <folder_path> -r <ref_file>
    OR
    python3 run_EBM_calculation.py -i <file_path> -o <folder_path> -r <ref_file>

EXIT STATUS
    Ctrl-C  =  cancel the script at any time

AUTHOR
    Dr. Marcel Buchhorn <marcel.buchhorn@vito.be>

LICENSE
    This script is property of VITO. Copyrights or restrictions may apply.

VERSION
    0.1 (2021-11-18)
"""

''' STILL a dummy '''

import pandas as pd
import geopandas as gpd
import os
import subprocess

def rasterize(gdb, path_out,column_name, nodata_value=0, dtype='Float32'):
    """ this function rasterize a given geopandas dataframe with a specified Column_name
         optional also give nodata value and dtype (dtype as string)"""
    # write the geodatabase to file can't use the general / not general enough
    if not os.path.exists(os.path.dirname(path_out)):
        os.makedirs(os.path.dirname(path_out))

    temp_out = os.path.join(os.path.dirname(path_out), 'vector_{}.shp'.format(os.path.basename(path_out).split('.')[0]))

    cmd = 'gdal_rasterize -a {} -l {} -init {} -a_nodata {} -co COMPRESS=LZW -ot {} -te {} {} {} {} -tr {} {} {} {}'.format(
        column_name,
        os.path.basename(temp_out).split('.shp')[0],
        nodata_value,
        nodata_value,
        dtype,
        "2525000.0",
        "1386700.0",
        "6575000.0",
        "5466700.0",
        25000.0,
        25000.0,
        temp_out,
        path_out)


    try:
        if not os.path.exists(os.path.dirname(temp_out)):
            os.makedirs(os.path.dirname(temp_out))
        gdb['i'] = gdb.index +1
        gdb.reset_index(drop=True,inplace=True)
        gdb.set_index('i', inplace=True)
        gdb.to_file(temp_out)
        subprocess.check_call(cmd, shell=True)
    except Exception:
        raise OSError('Could not rasterize needed vector file')





#input for AI
for year in [2000,2006,2012,2018,2021]:
    Monthly_Rad = pd.read_csv(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/Agri4Cast_preprocessed/unique_meanrad_{str(year-1)}_{str(year+1)}_monthly.csv')
    Monthly_MeanT = pd.read_csv(f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/Agri4Cast_preprocessed/unique_meantemp_{str(year-1)}_{str(year+1)}_monthly.csv')
    Monthly_daylength = pd.read_csv('/data/inca_vol1/input_data/crop_pollination/input_intermediate/Agri4Cast_preprocessed/avg_daylength.csv')
    names = ['Month_Code','CODE_NO']
    Monthly_Rad.index = pd.MultiIndex.from_frame(Monthly_Rad.iloc[:, 0:2], names=names)
    Monthly_MeanT.index = pd.MultiIndex.from_frame(Monthly_MeanT.iloc[:, 0:2], names=names)
    Monthly_daylength.index = pd.MultiIndex.from_frame(Monthly_daylength.iloc[:, 0:2], names=names)

    gdf = gpd.read_file('/data/inca_vol1/input_data/crop_pollination/input_raw/Agri4Cast/grid_25km.shp')
    all_info = pd.concat([Monthly_Rad, Monthly_MeanT, Monthly_daylength], axis=1)


    for month in range(1,13):
        output_rad = all_info.loc[([month], slice(None)), 'AVG_RAD' ]
        output_temp = all_info.loc[([month], slice(None)), 'Avg_T' ]
        output_day = all_info.loc[([month], slice(None)), 'avg_daylength' ]

        gdf_rad = gdf.merge(output_rad, how='left', left_on='Grid_Code', right_on='CODE_NO')
        gdf_temp = gdf.merge(output_temp, how='left', left_on='Grid_Code', right_on='CODE_NO')

        rasterize(gdf_rad, f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EBM/irradiation/{year}/{str(month).zfill(2)}_irradiation_{str(year)}_25km.tif', 'AVG_RAD')
        rasterize(gdf_temp, f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EBM/temperature/{year}/{str(month).zfill(2)}_temperature_{str(year)}_25km.tif', 'Avg_T')
        if year == 2000:
            gdf_day = gdf.merge(output_day, how='left', left_on='Grid_Code', right_on='CODE_NO')
            rasterize(gdf_day, f'/data/inca_vol1/input_data/crop_pollination/input_intermediate/EBM/day_length/{year}/{str(month).zfill(2)}_daylength_{str(year)}_25km.tif', 'avg_daylen')








