import os
import numpy as np
import pandas as pd
import subprocess
import rasterio
from rasterio.features import geometry_mask
from rasterio.transform import from_origin
from shapely.geometry import shape
from scipy.ndimage import convolve
from inca.common.classification import reclassification, CSV_2_dict
import inca.common.geoprocessing   as geo
i= 0
block_shape=(1024, 1024)
# Define file paths


id = '/data/inca_vol1/input_data/crop_pollination/input_intermediate/EBM'
if not os.path.exists(id):
    os.makedirs(id)


years = ['2006', '2012', '2018']


# Importing table with LC codes and scores
dict_classes = CSV_2_dict("/data/inca_vol1/input_data/crop_pollination/input_intermediate/input_jrc/Scores_LC.csv",'Value','Nat_area')
dict_classes2 = CSV_2_dict("/data/inca_vol1/input_data/crop_pollination/series_2018/input_preprocessing-intermediate/Look_up_Tables/crop-pollination_EBM-reclassification-schema.csv",
                           "old_class","new_class")

total_dict = {key: dict_classes[dict_classes2[key]] for key in dict_classes2 if dict_classes.get(dict_classes2[key])}
os.chdir(id)

# Reclassify input Land Cover to extract forest types (you need to prepare the reclassification matrix)
for year  in years:
    tif_name = f'nat_{year}.tif'
    with rasterio.open(f"/data/inca_vol1/input_data/update_series2021/general/CLCACC/CLC{year}ACC_V2018_20_split.tif") as src:
        profile = src.profile
        with rasterio.open(tif_name, 'w', **profile) as dst:
            for index, window in geo.block_window_generator(block_shape, profile['height'], profile['width']):
                rs = src.read(1, window = window)  # Reading a single band (assuming it's a single-band raster)
                profile = src.profile

                # Reclassify raster
                reclass, dict_classes = reclassification(rs, total_dict, np.nan, 0, outputtype=np.uint8 )

                # Save the reclassified raster as a GeoTIFF
                profile.update(dtype=np.uint8, nodata=0)
                dst.write(reclass, 1, window = window)

    #get nat_dist
    cmd = f"gdal_proximity.py -srcband 1 -distunits GEO -values 1 -maxdist 20000 -ot UInt16 -of GTiff "\
            f"{tif_name} natdist_{year}.tif"
    subprocess.check_call(cmd, shell=True)


print("end")
